BG = LibStub("AceAddon-3.0"):NewAddon("BG", "AceConsole-3.0", "AceEvent-3.0")

local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local LSM = LibStub:GetLibrary("LibSharedMedia-3.0")

local db

local options = {
    name = "BG",
    handler = BG,
    type = 'group',
    args = {
		General = {
			order = 1,
			type = "group",
			name = "General",
			desc = "General Settings",
			args = {
				intro = {
					order = 1,
					type = "description",
					name = "Version 0.1 - By Indz of Gorgonnash-EU",
				},
				general_settings = {
					name = "General Settings",
					type = 'group',
					inline = true,
					order = 7,
					args = {
						Locked = {
							order = 3,
							type = "toggle",
							name = "Locked",
							desc = "Lock/Unlock the frame",
							set = function(info,value)
								db.locked = value
							end,
							get = function(info)
								return db.locked
							end,
						},
						HideOOC = {
							order = 4,
							type = "toggle",
							name = "Hide out of combat",
							desc = "Hide/Show the frame when not in combat",
							set = function(info,value)
								db.hideooc = value
							end,
							get = function(info)
								return db.hideooc
							end,
						},
						Scale = {
							order = 5,
							type = "range",
							name = "Scale",
							min = .25, max = 3, step = .01,
							set = function(info,value)
								db.scale = value
							end,
							get = function(info)
								return db.scale
							end,
						},
						Alpha = {
							order = 6,
							type = "range",
							name = "Alpha",
							min = 0, max = 1, step = .01,
							set = function(info,value)
								db.alpha = value
							end,
							get = function(info)
								return db.alpha
							end,
						},
					},
				},
				fonts = {
					name = "Text appearance",
					type = 'group',
					inline = true,
					order = 7,
					args = {
						explanation = {
							name = "Set the text apprearance.",
							type = 'description',		
							order = 8,
						},
						font_face = {
							order = 9,
							type = "select",
							name = "Font",
							desc = "Font to use.",
							values = LSM:HashTable("font"),
							dialogControl = 'LSM30_Font',
        					set = function(info, value) 
								db.font_face = value
								BG:ResetFonts()
							end,
        					get = function() 
        						return db.font_face
        					end,
						},
						font_size = {
							order = 10,
							name = "Font size",
							desc = "Font size for the bars.",
							type = "range",
							min = 8, max = 30, step = 1,
        					set = function(info, value) 
								db.font_size = value
								BG:ResetFonts()
							end,
							get = function(info) 
								return db.font_size
							end,
						},
						font_length_pos = {
        					name = "Font Cycle Length Position",
        					desc = "Sets where the Cycle Length is displayed",
        					type = "select",
        					values = {
        					    ["TOPLEFT"] = "TOPLEFT",
        					    ["TOPRIGHT"] = "TOPRIGHT",
        					},
        					order = 11,
        					set = function(info, value)
        					    db.font_length_pos = value
    					        BG:UpdateFontPositions()
        					end,
                            get = function(info)
                                return db.font_length_pos
                            end,
        				},
					},
				},
				status_bar_settings = {
					name = "StatusBar setttings",
					type = 'group',
					inline = true,
					order = 11,
					args = {
						show_status_bar = {
							order = 11,
							type = "toggle",
							name = "Show",
							desc = "Show/Hide the StatusBar",
							set = function(info,value)
								db.show_status_bar = value
								BG:ToggleStatusBar()
							end,
							get = function(info)
								return db.show_status_bar
							end,
						},
						status_bar_texture = {
							order = 12,
							name = "Texture",
							desc = "StatusBarTextureDesc",
							type = "select",
							values = LSM:HashTable("statusbar"),
							dialogControl = 'LSM30_Statusbar',
							set = function(info, value)
								db.status_bar_texture = value
								BG:UpdateStatusBar()
							end,
        					get = function()
								return db.status_bar_texture
							end,
							disabled = function()
								return not db.show_status_bar
							end,
        				},
        			},
        		},
			},
		},
	},
}

local defaults = {
  profile = {
  	locked = false,
  	hideooc = false,
    alpha = 1,
    scale = 1,
    frame_x = 0,
    frame_y = 0,
    font_size = 12,
	font_face = "Friz Quadrata TT",
	font_length_pos = "TOPLEFT",
	show_status_bar = true,
	status_bar_texture = "Blizzard",
  }
}

-- local variables 
local inCombat = false
local lastTargetGUID = nil
local targetChanged = false
local BGState = 0
local BGCount = 0
local list = {}
local firstSS
local avg = 0

local function calcAvg()
	local last = 0
	local diff
	avg = 0
	for i,v in pairs(list) do
        diff = v-last
		avg = ((i-1)*avg+diff)/i
		last = v
    end
end

local function addTime()
	if not firstSS then
		firstSS = GetTime()
	else
		table.insert(list,GetTime()-firstSS)
	end
	calcAvg()
end

local function getTimeLeftOnBG3()
	local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID
	local unit = "player"
	local filter = "HELPFUL"
	for index=1,40 do
		name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID = UnitAura(unit, index ,filter)
		if spellID == 84747 then
			return (expires-GetTime())
		end
	end
	return 0
end

local function getActualPoint()
	local stage = BGState -- 0,1,2,3
	local intermediatestage
	if stage ~= 3 then
		intermediatestage = BGCount
		point = (4-intermediatestage)*avg+(2-stage)*4*avg+15
	else
		point = getTimeLeftOnBG3()
	end
	return point
end

local function getTimeToCycleEnd()
	local point = getActualPoint()
	local maximum = avg*4+15
	
	local left = point
	
	--print("cycle ends in " .. left)
	return left
end

function BG:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BGDB", defaults, "Default")
	
	db = self.db.profile
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileUpdate")
	
	ACR:RegisterOptionsTable("BG", options)
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	self.InterfaceOptions = ACD:AddToBlizOptions("BG", "BG", nil, "General")

	ACD:AddToBlizOptions("BG", "Profiles", "BG", "profiles")
	LibStub("LibAboutPanel").new("BG", "BG")
		
	self:RegisterChatCommand("bgh", "ChatCommand")
end


function BG:OnEnable()
		
	BG:Print("BG Helper loaded!")
    
	BG:RegisterEvent("UNIT_AURA")
	BG:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	BG:RegisterEvent("PLAYER_TARGET_CHANGED")
	BG:RegisterEvent("PLAYER_REGEN_DISABLED")
	BG:RegisterEvent("PLAYER_REGEN_ENABLED")
	BG:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnTalentChanged")
	BG:RegisterEvent("PLAYER_TALENT_UPDATE", "OnTalentChanged")
	
	-- LDB launcher
	if LDB then
		BGDS = LDB:NewDataObject("BG", {
			icon = "Interface\\Icons\\INV_Bijou_Red",
			label = "BGHelper",
			text = "BG Helper",
			type = "data source",
			OnClick = function(self, button)
				if button == "RightButton" then
					--LoggerHead:ShowConfig()
					BG:Print("rightclick")
					BG:ChatCommand()
				end
				if button == "LeftButton" then
					--if LoggingCombat() then
					--	LoggerHead:DisableLogging()
					--else
					--	LoggerHead:EnableLogging()
					--end
					BG:Print("leftclick")
					
				end
			end,
			OnTooltipShow = function(tooltip)
				tooltip:AddLine("BGHelper")
				tooltip:AddLine(" ")
				tooltip:AddLine("Click to toggle on/off")
				tooltip:AddLine("Right-click to open the options menu")
			end
		})
	end
	
	local f = CreateFrame("Button","BG_Display",UIParent,"BG_Display")
	f:RegisterForDrag("LeftButton")
	f:RegisterForClicks("AnyUp")
	f:SetPoint("CENTER",UIParent,"CENTER",db.frame_x,db.frame_y)
	f:Show()
	f:SetScript("OnDragStart", function(self) if(not db.locked) then self:StartMoving() end end)
	f:SetScript("OnDragStop", function(self)
									self:StopMovingOrSizing() 
									local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
									local x, y = self:GetCenter()
									x, y = x * scale, y * scale
									x = x - GetScreenWidth()/2
									y = y - GetScreenHeight()/2
									x = x / self:GetScale()
									y = y / self:GetScale()
									db.frame_x = x
									db.frame_y = y
								end)
	f:SetScript("OnUpdate", function(self) BG:update(self) end)
	BG:ResetFonts()
	BG:UpdateStatusBar()
	BG:ToggleStatusBar()
	BG:UpdateFontPositions()
end

function BG:OnDisable()
    -- Called when the addon is disabled
end

function BG:update(f)
	-- update
	
	f:SetScale(db.scale)
	f:SetAlpha(db.alpha)
	f.count:SetText(BGCount)
	if BGState == 3 then
		f.count:SetText("")
	end
	f.left:SetText(math.floor(getTimeToCycleEnd()))
	f.length:SetText(math.floor(avg*4*3+15))
	
	f.bar:SetMinMaxValues(0, math.floor(avg*4*3+15) )
	f.bar:SetValue( math.floor(getTimeToCycleEnd()) )
	
	if BGState == 1 then
		f.bar:SetStatusBarColor(0.1,1,0.1,db.alpha)
		f.bar.bg:SetVertexColor(0.1,1,0.1,db.alpha/4)
		f.border:SetVertexColor(0.1,1,0.1,db.alpha/4)
	elseif BGState == 2 then
		f.bar:SetStatusBarColor(1,1,0,db.alpha)
		f.bar.bg:SetVertexColor(1,1,0,db.alpha/4)
		f.border:SetVertexColor(1,1,0,db.alpha/4)
	elseif BGState == 3 then
		f.bar:SetStatusBarColor(1,0.1,0.1,db.alpha)
		f.bar.bg:SetVertexColor(1,0.1,0.1,db.alpha/4)
		f.border:SetVertexColor(1,0.1,0.1,db.alpha/4)
	else
		f.bar:SetStatusBarColor(0.5,0.5,0.5,db.alpha)
		f.bar.bg:SetVertexColor(0.5,0.5,0.5,db.alpha/4)
		f.border:SetVertexColor(0.5,0.5,0.5,db.alpha/4)
	end
	
end

function BG:UNIT_AURA(eventName, ...)
	local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID
	local unit = "player"
	local filter = "HELPFUL"
	for index=1,40 do
		name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID = UnitAura(unit, index, filter)
		if spellID == 84745 then
			if BGState ~= 1 then
				BGState = 1
				BGCount = 0
				_G["BG_Display"].icon:SetTexture(icon)
			end
			return
		elseif spellID == 84746 then
			if BGState ~= 2 then
				BGState = 2
				BGCount = 0
				_G["BG_Display"].icon:SetTexture(icon)
			end
			return
		elseif spellID == 84747 then
			if BGState ~= 3 then
				BGState = 3
				BGCount = 0
				_G["BG_Display"].icon:SetTexture(icon)
			end
			return
		end
	end
	_G["BG_Display"].icon:SetTexture(nil)
	BGState = 0
end

function BG:COMBAT_LOG_EVENT_UNFILTERED(eventName, ...)
	local arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,arg11,arg12,arg13 = ...;
	if arg4 ~= UnitName("player") then
		return
	end
	if arg2 == "SPELL_DAMAGE" and (arg9 == 1752 or arg9 == 84617) then
		if not targetChanged or not lastTargetGUID then
			lastTargetGUID = UnitGUID("target")
		end
		if targetChanged and lastTargetGUID then
			--BG:Print("lasttarget: " .. lastTargetGUID)
			--BG:Print("target: " .. UnitGUID("target"))
			if lastTargetGUID  ~= UnitGUID("target") then
				BGState = 0
				BGCount = 0
				lastTargetGUID = nil
				targetChanged = false
			else
				targetChanged = false
			end
		end
				
		BGCount = BGCount + 1
		addTime()
		--print("State: " .. BGState .. " " .. BGCount)
	end
	if arg2 == "SPELL_AURA_REMOVED" and arg9 == 84747 then
		BGCount = 0
		--print("Cycle starts anew.")
	end
	if arg2 == "SPELL_CAST_SUCCESS" and arg9 == 73981 then
		targetChanged = false
	end
end

function BG:PLAYER_TARGET_CHANGED(eventName, ...)
	if UnitExists("target") then
		targetChanged = true
	end
end

function BG:PLAYER_REGEN_DISABLED(eventName, ...)
	inCombat = true
end

function BG:PLAYER_REGEN_ENABLED(eventName, ...)
	inCombat = false
	BGState = 0
	BGCount = 0
	list = {}
	firstSS = nil
	avg = 0
end

function BG:OnTalentChanged(eventName, ...)
	--BG:Print("talents changed")
	--BG:OnEnable()
	local f = _G["BG_Display"]
	if not BG:isCombatRogue() then
		f:Hide()
	else
		f:Show()
	end
end

function BG:isCombatRogue()
	local class, classFileName 
	local name, iconTexture, tier, column, rank, maxRank, isExceptional, meetsPrereq, previewRank, meetsPreviewPrereq
	class, classFileName = UnitClass("player")
	if classFileName ~= "ROGUE" then
		return false
	end
	name, iconTexture, tier, column, rank, maxRank, isExceptional, meetsPrereq, previewRank, meetsPreviewPrereq = GetTalentInfo(2, 17)
	if name then
		if rank == 3 then
			return true
		end
	end
	return false
end

function BG:ProfileUpdate()
    db = self.db.profile
    _G["BG_Display"]:ClearAllPoints()
	_G["BG_Display"]:SetPoint("CENTER",UIParent,"CENTER",db.frame_x,db.frame_y)
	BG:ResetFonts()
	BG:UpdateStatusBar()
	BG:ToggleStatusBar()
	BG:UpdateFontPositions()
end

function BG:ResetFonts()
	local f = _G["BG_Display"]
	local fontName, fontHeight, fontFlags = f.left:GetFont()
	local ff = LSM:Fetch("font",db.font_face)
	local fh = db.font_size
	f.count:SetFont(ff,fh,fontFlags)
	f.count:SetText(f.count:GetText())
	f.left:SetFont(ff,fh,fontFlags)
	f.left:SetText(f.left:GetText())
	f.length:SetFont(ff,fh,fontFlags)
	f.length:SetText(f.length:GetText())
end

function BG:UpdateStatusBar()
	local bar = _G["BG_Display"].bar
	local bt = LSM:Fetch("statusbar",db.status_bar_texture)
	bar:SetStatusBarTexture(bt)
	bar.bg:SetTexture(bt)
	--bar:GetStatusBarTexture():SetHorizTile(false)
	--bar:GetStatusBarTexture():SetVertTile(false)
end

function BG:ToggleStatusBar()
	local bar = _G["BG_Display"].bar
	if db.show_status_bar then
		bar:Show()
	else
		bar:Hide()
	end
end

function BG:UpdateFontPositions()
	local f = _G["BG_Display"]
	f.length:ClearAllPoints()
	f.length:SetPoint(db.font_length_pos,f,db.font_length_pos,0,0)
end

function BG:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.InterfaceOptions)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(BG, "bg", "BG", input)
    end
end




